/**
 * This function uses the modified Lentz's method for solving continued fractions of the
 * form :
 *               a1   a2   a3   a4   a5
 *          b0 + ---  ---  ---  ---  --- . . .
 *               b1+  b2+  b3+  b4+  b5+
 *
 * for a description of the algorithm, visit the web page:
 *      http://www.aip.de/groups/soe/local/numres/bookcpdf/c5-2.pdf
 *
 *
 * @param a - a function that has one argument, j, that gives the a values in the
 *          continued fraction representation above for j = 1, 2, 3, ....
 * @param b - a function that has one argument, j, that gives the b values in the
 *          continued fraction representation above for j = 1, 2, 3, ....
 * @returns {*} - the estimated value of the continued fraction
 */
export declare function continuedFractionSolver(a: (j: number) => number, b: (j: number) => number): number;
